local util = require("util")

-- Code to run whenever a turret is created, or on existing turrets when this mod is first loaded
function initialise_turret(turret)
	turret.disabled_by_script = true
	local id = turret.unit_number
	storage.entities[id] = turret
	storage.power_entities[id] = turret.surface.create_entity
	{
		name = turret.name.."-power",
		position = turret.position,
		force = turret.force,
		direction = turret.direction,
		destructible = false
	}
end

function destroy_turret(id) -- Can't actually pass the turret Entity object here as it may be invalid
    if storage.power_entities[id] then
		storage.power_entities[id].destroy()
		storage.power_entities[id] = nil
		storage.entities[id] = nil
	end
end

---- INIT ----
script.on_init(
function()
	storage.entities = {}
	storage.power_entities = {}

	for _, turret_type in pairs{"artillery-turret", "flamethrower-turret"} do
		for _, surface in pairs(game.surfaces) do
			for _, turret in pairs(surface.find_entities_filtered{name = turret_type}) do
				initialise_turret(turret)
			end
		end
	end
end)

---- ON EVENT ----
script.on_event({defines.events.on_robot_built_entity, defines.events.on_built_entity}, 
function(event)
	if event.entity.name == "artillery-turret" or event.entity.name == "flamethrower-turret" then
		initialise_turret(event.entity) end
end)

script.on_event({defines.events.on_entity_died, defines.events.on_robot_pre_mined, defines.events.on_pre_player_mined_item, defines.events.on_space_platform_pre_mined},
function(event)
	destroy_turret(event.entity.unit_number)
end)

local refresh_rate = 1 -- how often to check whether an out-of-power turret should be re-enabled
local energy_per_shot = {
	["artillery-turret"] = util.parse_energy("8kJ"), -- set to whatever you want
	["flamethrower-turret"] = util.parse_energy("1kJ") }

script.on_event(defines.events.on_script_trigger_effect, 
function (event)
	if event.effect_id ~= "turret-consume-energy" then
		return
	end
	local turret = event.cause_entity -- or should it be event.source_entity?
	local id = turret.unit_number
	local interface = storage.power_entities[id]
	if interface.energy >= energy_per_shot[turret.name] then
		interface.energy = interface.energy - energy_per_shot[turret.name]
	else
		turret.disabled_by_script = true
	end
end)

script.on_nth_tick(refresh_rate,
function(event)
    for id, interface in pairs(storage.power_entities) do
		local turret = storage.entities[id]
		if not turret.valid then
			destroy_turret(id)
		elseif turret.disabled_by_script and interface.energy >= energy_per_shot[turret.name] then
			interface.energy = interface.energy - energy_per_shot[turret.name]
			turret.disabled_by_script = false
		end
	end
end)