local resistances_immune = {}
for name, _ in pairs(data.raw["damage-type"]) do -- make sure we get every possible damage type
	table.insert(resistances_immune, {type = name, percent = 100}) end

-- Loop over each turret we are adding energy to
for v, energy in pairs
	{
		[data.raw["artillery-turret"]["artillery-turret"]] =
		{
			type = "electric",
			buffer_capacity = "50kJ",
			input_flow_limit = "60kW",
			usage_priority = "primary-input",
			drain = "50kW"
		},
		[data.raw["fluid-turret"]["flamethrower-turret"]] =
		{
			type = "electric",
			buffer_capacity = "20kJ",
			input_flow_limit = "30kW",
			usage_priority = "primary-input",
			drain = "20kW"
		}
	} do
	data:extend
	{
		{
			type = "electric-energy-interface",
			name = v.name.."-power",
			-- Example: Flamethrower turret (Energy)
			localised_name = {"", v.localised_name or {"entity-name." .. v.name}, " (", {"description.energy"} ,")"},
			icon = v.icon,
			icon_size = v.icon_size, icons = v.icons,
			flags = {"not-on-map", "player-creation", "not-deconstructable","not-blueprintable", "not-upgradable"},
			selection_priority = (v.selection_priority or 50) + 1, -- so hovering the cursor over the power interface and turret will select the interface
			selection_box = {{-0.5, 0.5}, {0.5, 1.5}}, -- square at the bottom middle
			max_health = 1,
			resistances = resistances_immune,
			hidden = true,
			gui_mode = "none",
			collision_box = v.collision_box,
			energy_source = energy,
		}
	}
end

-- tbl[field] should be nil, a single Trigger, or an array of Trigger's
-- (Also works for TriggerDelivery's, and TriggerEffect's)
-- get_trigger will either return on of the triggers, or return default_trigger, after storing it in tbl[field]
function get_trigger(tbl, field, default_trigger)
	if not tbl[field] then -- no Triggers
		tbl[field] = default_trigger
		return tbl[field]
	elseif tbl[field].type then -- not an array
		return tbl[field]
	elseif #tbl[field] > 0 then -- non-empty array of Triggers
		return tbl[field][#tbl[field]] -- use the last Trigger
	else -- should be an empty array
		table.insert(tbl[field], default_trigger)
		return tbl[field][1] 
	end 
end

function insert_trigger(tbl, field, new_trigger)
	if not tbl[field] then
		tbl[field] = new_trigger
	elseif tbl[field].type then -- not an array, turn it into one
		tbl[field] = {tbl[field], new_trigger}
	else -- should be an array
		tbl.insert(tbl[field], new_trigger)
	end
	return new_trigger
end



for _, obj in pairs{data.raw["ammo"]["artillery-shell"].ammo_type, data.raw["stream"]["flamethrower-fire-stream"]} do
	local action = get_trigger(obj, "action", {type = "direct"})
	local action_delivery = get_trigger(action, "action_delivery", {type = "instant"})
	insert_trigger(action_delivery, "source_effects", {type = "script", effect_id = "turret-consume-energy"}) end